<?php

namespace App\Console\Commands\Intuit;

use Illuminate\Console\Command;
use Intuit\Jobs\AccountingCustomerUpdate;
use Intuit\Models\Customer as AccAccount;

use App\Models\{Account,ProviderOauth,User};

class AccountAdd extends Command
{
	private const provider = 'intuit';

	/**
	 * The name and signature of the console command.
	 *
	 * @var string
	 */
	protected $signature = 'intuit:account:add'
		.' {id : Account ID}'
		.' {user? : User Email}';

	/**
	 * The console command description.
	 *
	 * @var string
	 */
	protected $description = 'Add an account to quickbooks';

	/**
	 * Execute the console command.
	 *
	 * @return int
	 */
	public function handle()
	{
		$uo = User::where('email',$this->argument('user') ?: config('osb.admin'))->singleOrFail();

		$so = ProviderOauth::where('name',self::provider)->singleOrFail();
		if (! ($to=$so->token($uo)))
			abort(500,sprintf('Unknown Tokens for [%s]',$uo->email));

		$o = Account::findOrFail($this->argument('id'));

		$acc = new AccAccount;
		$acc->PrimaryEmailAddr = (object)['Address'=>$o->user->email];
		$acc->ResaleNum = $o->sid;
		$acc->GivenName = $o->user->firstname;
		$acc->FamilyName = $o->user->lastname;
		$acc->CompanyName = $o->name;
		$acc->DisplayName = $o->name;
		$acc->FullyQualifiedName = $o->name;
		$acc->Active = (bool)$o->active;

		return AccountingCustomerUpdate::dispatchSync($to,$acc);
	}
}