<?php

namespace App\Console\Commands\Intuit;

use GuzzleHttp\Exception\ConnectException;
use Illuminate\Console\Command;
use Intuit\Exceptions\ConnectionIssueException;

use App\Models\{ProviderOauth,User};

class AccountGet extends Command
{
	private const provider = 'intuit';

	/**
	 * The name and signature of the console command.
	 *
	 * @var string
	 */
	protected $signature = 'intuit:account:get'
		.' {id : Account ID}'
		.' {user? : User Email}';

	/**
	 * The console command description.
	 *
	 * @var string
	 */
	protected $description = 'Get an account from quickbooks';

	/**
	 * Execute the console command.
	 *
	 * @return int
	 */
	public function handle()
	{
		$uo = User::where('email',$this->argument('user') ?: config('osb.admin'))->singleOrFail();

		$so = ProviderOauth::where('name',self::provider)->singleOrFail();
		if (! ($to=$so->token($uo)))
			abort(500,sprintf('Unknown Tokens for [%s]',$uo->email));

		try {
			$api = $to->API();
			dump($api->getAccountQuery($this->argument('id')));

		} catch (ConnectException|ConnectionIssueException $e) {
			$this->error($e->getMessage());

			return self::FAILURE;
		}

		return self::SUCCESS;
	}
}