<?php

namespace App\Console\Commands\Intuit;

use GuzzleHttp\Exception\ConnectException;
use Illuminate\Console\Command;
use Intuit\Exceptions\ConnectionIssueException;

use App\Jobs\AccountingPaymentSync as Job;
use App\Models\{ProviderOauth,User};

class PaymentGet extends Command
{
	private const provider = 'intuit';

	/**
	 * The name and signature of the console command.
	 *
	 * @var string
	 */
	protected $signature = 'accounting:payment:get'
		.' {id : Payment ID}'
		.' {user? : User Email}';

	/**
	 * The console command description.
	 *
	 * @var string
	 */
	protected $description = 'Get a payment from the accounting provider';

	/**
	 * Execute the console command.
	 *
	 * @return int
	 */
	public function handle()
	{
		$uo = User::where('email',$this->argument('user') ?: config('osb.admin'))->singleOrFail();

		$so = ProviderOauth::where('name',self::provider)->singleOrFail();
		if (! ($to=$so->token($uo)))
			abort(500,sprintf('Unknown Tokens for [%s]',$uo->email));

		try {
			$api = $to->API();
			$acc = $api->getPayment($this->argument('id'));
			dump($acc);

		} catch (ConnectException|ConnectionIssueException $e) {
			$this->error($e->getMessage());

			return Command::FAILURE;
		}

		if ($acc)
			Job::dispatchSync($to,$acc);

		return self::SUCCESS;
	}
}